package databasemodels

import "gorm.io/gorm"

type User struct {
	gorm.Model
	Id               string `json:"id"`
	DisplayName      string `json:"display_name"`
	Username         string `json:"user_name"`
	Avatar           string `json:"avatar"`
	AvatarDecoration string `json:"avatar_decoration"`
	LoginToken       string `json:"login_token"`
	LoggedIn         bool   `json:"logged_in"`
}

type Person struct {
	gorm.Model
	Name   string  `json:"name"`
	Groups []Group `gorm:"many2many:person_group_associations" json:"groups"` // Unique
}

type Group struct {
	gorm.Model
	Name string `json:"name"`
}

type Character struct {
	gorm.Model
	Name         string          `gorm:"uniqueIndex" json:"name"`
	Owners       []Person        `gorm:"many2many:character_owner_associations" json:"owners"` // Unique
	Roles        []Role          `gorm:"many2many:character_role_associations" json:"roles"`   // Unique
	FunctionSets []FunctionSet   `gorm:"foreignkey:id; references:Name" json:"function_sets"`
	Inventory    []InventorySlot `gorm:"foreignkey:id; references:Name" json:"inventory"`
}

type Role struct {
	gorm.Model
	Name       string  `json:"name"`
	Tiers      []Tier  `gorm:"foreignkey:id; references:Name" json:"tiers"`
	Visibility []Group `gorm:"foreignkey:Name" json:"visibility"` // Unique
}

type Tier struct {
	gorm.Model
	FunctionSets []FunctionSet `gorm:"foreignkey:id; references:id" json:"function_sets"`
}

type FunctionSet struct {
	gorm.Model
	Functions []Function `gorm:"foreignkey:Name" json:"functions"`
}

type Function struct {
	gorm.Model
	Name         string        `gorm:"primaryKey" gorm:"uniqueIndex" json:"name"`
	Tags         []FunctionTag `gorm:"many2many:function_tag_associations" json:"tags"`
	Requirements []Function    `gorm:"many2many:function_requirement_associations" json:"requirements"`
}

type FunctionTag struct {
	gorm.Model
	Name string `gorm:"primaryKey" gorm:"uniqueIndex" json:"name"`
}

type InventorySlot struct {
	gorm.Model
	Item     Item  `gorm:"foreignkey:Name" json:"item"`
	Quantity int64 `json:"quantity"` // Positive
}

type Item struct {
	gorm.Model
	Name                string          `json:"name"`
	Functions           []Function      `gorm:"foreignkey:Name" json:"functions"`
	FlavorText          string          `json:"flavor_text"`
	RulesDescription    string          `json:"rules_description"`
	PhysrepRequirements string          `json:"physrep_requirements"`
	Tags                []ItemTag       `gorm:"foreignkey:Name" json:"tags"` // Unique
	Customizations      []Customization `gorm:"foreignkey:Name" json:"customizations"`
	Visibility          []Group         `gorm:"foreignkey:Name" json:"visibility"` // Unique
}

type ItemTag struct {
	gorm.Model
	Name string `gorm:"primaryKey" json:"name"`
}

type Customization struct {
	gorm.Model
	Name                string     `json:"name"`
	Functions           []Function `gorm:"foreignkey:Name" json:"functions"`
	FlavorText          string     `json:"flavor_text"`
	RulesDescription    string     `json:"rules_description"`
	PhysrepRequirements string     `json:"physrep_requirements"`
	Tags                []ItemTag  `gorm:"foreignkey:Name" json:"tags"`       // Unique
	Visibility          []Group    `gorm:"foreignkey:Name" json:"visibility"` // Unique
}

type Schematic struct {
	gorm.Model
	Material     []InventorySlot `gorm:"many2many:schematic_material_associations" json:"material"`
	Tools        []InventorySlot `gorm:"many2many:schematic_tool_associations" json:"tools"`
	Requirements []Function      `gorm:"many2many:schematic_requirement_associations" json:"requirements"`
	TimeUnits    int64           `json:"time_units"` // Positive
	Result       InventorySlot   `gorm:"many2many:schematic_result_associations" json:"result"`
	Visibility   []Group         `gorm:"many2many:schematic_group_associations" json:"visibility"` // Unique
}
